﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using Rss;
using Rhino.Mocks;

namespace MockSubSonic.Tests
{
    [TestFixture]
    public class DatabaseManagerTests
    {
        RssFeed rssFeed;
        MockRepository mocks;
        IDataStore database;

        [SetUp]
        public void TestSetup()
        {
            rssFeed = TestHelper.CreateMockRSS(TestHelper.url);
            mocks = new MockRepository();
            database = mocks.CreateMock<IDataStore>();
            DatabaseManager.DataStore = database;
        }

        [TearDown]
        public void TestTearDown()
        {
            DatabaseManager.DataStore = null;
            rssFeed = null;
            mocks = null;
            database = null;
        }

        [Test]
        public void InsertFeedIntoDatabase()
        {
            int channelID = 1;

            using (mocks.Record())
            {
                Expect.Call(database.InsertFeedDetails(rssFeed.Channels[0])).Return(channelID);
                Expect.Call(database.InsertFeedItem(channelID, rssFeed.Channels[0].Items[0])).Return(1);
                Expect.Call(database.InsertFeedItem(channelID, rssFeed.Channels[0].Items[1])).Return(2);
            }

            using (mocks.Playback())
            {
                bool result = DatabaseManager.SaveFeed(rssFeed);
                Assert.IsTrue(result);
            }
        }

        [Test]
        [ExpectedArgumentNullException]
        public void ThrowExceptionOnSaveFeedIfNoDataSource()
        {
            DatabaseManager.DataStore = null;
            DatabaseManager.SaveFeed(rssFeed);
        }

        /// <summary>
        /// This test verifies that the DatabaseManager correctly communicates with the DataStore.
        /// </summary>
        [Test]
        public void InsertFeedDetailsIntoDatabase()
        {
            using (mocks.Record())
            {
                Expect.Call(database.InsertFeedDetails(rssFeed.Channels[0])).Return(1);
            }

            using (mocks.Playback())
            {
                int recordID = DatabaseManager.InsertFeedDetails(rssFeed.Channels[0]);

                Assert.GreaterThan(recordID, 0);
            }
        }

        [Test]
        public void InsertFeedItemsIntoDatabase()
        {
            int channelId = 1;
            using (mocks.Record())
            {
                Expect.Call(database.InsertFeedItem(channelId, rssFeed.Channels[0].Items[0])).Return(1);
                Expect.Call(database.InsertFeedItem(channelId, rssFeed.Channels[0].Items[1])).Return(2);
            }

            using (mocks.Playback())
            {
                bool result = DatabaseManager.InsertFeedItems(channelId, rssFeed.Channels[0].Items);
                Assert.IsTrue(result);
            }
        }

        [Test]
        public void GetChannelFromDatabase()
        {
            using (mocks.Record())
            {
                Expect.Call(database.GetRssChannel(TestHelper.url)).Return(rssFeed.Channels[0]);
            }

            using (mocks.Playback())
            {
                RssChannel rssChannel = DatabaseManager.GetRssChannel(TestHelper.url);
                Assert.IsNotNull(rssChannel);
                Assert.AreEqual(TestHelper.url, rssChannel.Link.AbsoluteUri);
            }
        }

        [Test]
        public void GetFeedItemsForChannelFromDatabase()
        {
            int channelId = 1;
            using (mocks.Record())
            {
                Expect.Call(database.GetRssItemsForChannel(channelId)).Return(rssFeed.Channels[0].Items);
            }

            using (mocks.Playback())
            {
                RssItemCollection rssFeeds = DatabaseManager.GetRssFeedsForChannel(channelId);
                Assert.IsNotNull(rssFeeds);
                Assert.AreEqual(2, rssFeeds.Count);
            }
        }

        [Test]
        public void GetRssFromDatabase()
        {
            int channelId = 1;

            //This is to clone the collection so we can use it cleanly in our mock object
            //Sadly they aren't ICloneable.
            RssItemCollection feedItems = new RssItemCollection();
            foreach (RssItem item in rssFeed.Channels[0].Items)
            {
                RssItem rss = new RssItem();
                rss.Title = item.Title;
                rss.PubDate = item.PubDate;
                rss.Link = item.Link;
                rss.Description = item.Description;
                rss.Author = item.Author;
                feedItems.Add(rss);
            }

            //Ensure correct mock results.
            RssChannel cleaned = rssFeed.Channels[0];
            cleaned.Items.Clear();

            using (mocks.Record())
            {
                Expect.Call(database.GetRssChannel(TestHelper.url)).Return(cleaned);
                Expect.Call(database.GetChannelIDForUrl(TestHelper.url)).Return(channelId);
                Expect.Call(database.GetRssItemsForChannel(channelId)).Return(feedItems);
            }

            using (mocks.Playback())
            {
                RssFeed actualFeed = DatabaseManager.GetFeed(TestHelper.url);
                Assert.IsNotNull(actualFeed);
                Assert.AreEqual(1, actualFeed.Channels.Count);
                Assert.AreEqual(2, actualFeed.Channels[0].Items.Count);
            }
        }

        [Test]
        public void SaveUser()
        {
            User user = TestHelper.CreateUser();

            using (mocks.Record())
            {
                Expect.Call(database.SaveUser(user)).Return(1);
            }

            using (mocks.Playback())
            {
                int userId = DatabaseManager.SaveUser(user);
                Assert.GreaterThan(userId, 0);
            }
        }

        [Test]
        public void UserSubscribesToChannel()
        {
            User user = TestHelper.CreateUser();
            int channelId = 1;

            using (mocks.Record())
            {
                Expect.Call(database.GetChannelIDForUrl(TestHelper.url)).Return(channelId);
                Expect.Call(database.Subscribe(user.Name, channelId)).Return(true);
            }

            using (mocks.Playback())
            {
                bool result = DatabaseManager.SubscribeUserToFeed(user.Name, TestHelper.url);
                Assert.IsTrue(result);
            }
        }

        [Test]
        public void GetUserRssFeedCollection()
        {
#region SetupCode
            User user = TestHelper.CreateUser();
            int userId = 1;
            int channelId = 1;
            int[] channelList = new int[] { channelId };
            
            RssItemCollection feedItems = new RssItemCollection();
            foreach (RssItem item in rssFeed.Channels[0].Items)
            {
                RssItem rss = new RssItem();
                rss.Title = item.Title;
                rss.PubDate = item.PubDate;
                rss.Link = item.Link;
                rss.Description = item.Description;
                rss.Author = item.Author;
                feedItems.Add(rss);
            }

            RssChannel cleaned = rssFeed.Channels[0];
            cleaned.Items.Clear();

#endregion

            using (mocks.Record())
            {
                Expect.Call(database.GetUserIDForUsername(user.Name)).Return(userId);
                Expect.Call(database.GetChannelListForUser(userId)).Return(channelList);
                Expect.Call(database.GetRssChannel(channelId)).Return(cleaned);
                Expect.Call(database.GetRssItemsForChannel(channelId)).Return(feedItems);
            }

            using (mocks.Playback())
            {
                RssFeedCollection result = DatabaseManager.GetUsersRssFeedCollection(user.Name);
                Assert.AreEqual(1, result.Count);
                Assert.AreEqual(1, result[0].Channels.Count);
                Assert.AreEqual(2, result[0].Channels[0].Items.Count);
            }
        }
    }
}
