﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using Rss;
using System.Data.SqlClient;

namespace MockSubSonic.Tests
{
    [TestFixture]
    public class SubSonicTests
    {
        RssFeed rss;
        SubSonicDataStore dataSource;

        [SetUp]
        public void TestSetup()
        {
            rss = TestHelper.CreateMockRSS(TestHelper.url);
            dataSource = new SubSonicDataStore();
            ExecuteSql("TRUNCATE TABLE RssChannel");
            ExecuteSql("TRUNCATE TABLE RssItem");
            ExecuteSql("TRUNCATE TABLE UserChannelSubscription");
            ExecuteSql("TRUNCATE TABLE [User]");
        }

        [TearDown]
        public void TestTearDown()
        {
            rss = null;
            dataSource = null;
            ExecuteSql("TRUNCATE TABLE RssChannel");
            ExecuteSql("TRUNCATE TABLE RssItem");
            ExecuteSql("TRUNCATE TABLE UserChannelSubscription");
            ExecuteSql("TRUNCATE TABLE [User]");
        }

        private static void ExecuteSql(string sql)
        {
            SqlConnection sqlConn = new SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["DatabaseConnection"].ConnectionString);
            SqlCommand sqlCmd = new SqlCommand(sql, sqlConn);

            try
            {
                sqlConn.Open();

                sqlCmd.ExecuteNonQuery();
            }
            finally
            {
                if (sqlConn.State == System.Data.ConnectionState.Open)
                    sqlConn.Close();
            }
        }
        
        [Test]
        public void ConvertFromRssChannelToSubSonicRssChannel()
        {
            RssChannel rssChannel = rss.Channels[0];
            //Returns a SubSonic object. 
            MockSubSonic.DAL.RssChannel subSonicChannelObject = dataSource.ConvertRssChannelObject(rssChannel);
            Assert.AreEqual(rssChannel.Title, subSonicChannelObject.Title);
            Assert.AreEqual(TestHelper.url, rssChannel.Link.AbsoluteUri);
            Assert.AreEqual(rssChannel.Link.AbsoluteUri, subSonicChannelObject.Uri);
            Assert.AreEqual(rssChannel.Description, subSonicChannelObject.Description);
            Assert.AreEqual(rssChannel.PubDate, subSonicChannelObject.PubDate);
            Assert.IsTrue(subSonicChannelObject.IsNew); //it shouldn't save it.
        }

        [Test]
        public void ConvertFromRssIemToSubSonicItem()
        {
            RssItem rssItem = rss.Channels[0].Items[0];
            MockSubSonic.DAL.RssItem subSonicRssItem = dataSource.ConvertRssItemObject(rssItem);
            Assert.AreEqual(rssItem.Title, subSonicRssItem.Title);
            Assert.AreEqual(rssItem.PubDate, subSonicRssItem.PubDate);
            Assert.AreEqual(rssItem.Link, subSonicRssItem.Link);
            Assert.AreEqual(rssItem.Description, subSonicRssItem.Description);
            Assert.AreEqual(rssItem.Author, subSonicRssItem.Author);
            Assert.IsTrue(subSonicRssItem.IsNew); //it shouldn't save it.          
        }

        [Test]
        public void InsertRssChannelIntoDatabase()
        {
            int channelId = dataSource.InsertFeedDetails(rss.Channels[0]);
            Assert.AreEqual(channelId, 1);
        }

        [Test]
        public void InsertRssItemIntoDatabase()
        {
            int channelId = dataSource.InsertFeedDetails(rss.Channels[0]);
            Assert.GreaterEqualThan(channelId, 0);

            int itemId = dataSource.InsertFeedItem(channelId, rss.Channels[0].Items[0]);
            Assert.AreEqual(itemId, 1);
        }

        [Test]
        public void GetRssChannel()
        {
            dataSource.InsertFeedDetails(rss.Channels[0]);

            RssChannel rssChannel = dataSource.GetRssChannel(TestHelper.url);

            //As we are not serialising the object and only storing interesting information, we need just to assert that information.
            Assert.AreEqual(rss.Channels[0].Title, rssChannel.Title);
            Assert.AreEqual(rss.Channels[0].PubDate.ToString(), rssChannel.PubDate.ToString()); //Strings are the same, ticks are different due to lag in SQL Server
            Assert.AreEqual(rss.Channels[0].Description, rssChannel.Description);
            Assert.AreEqual(rss.Channels[0].Link.AbsoluteUri, rssChannel.Link.AbsoluteUri);
        }

        [Test]
        public void GetRssChannelFeeds()
        {
            int channelId = dataSource.InsertFeedDetails(rss.Channels[0]);
            dataSource.InsertFeedItem(channelId, rss.Channels[0].Items[0]);
            dataSource.InsertFeedItem(channelId, rss.Channels[0].Items[1]);

            RssItemCollection rssColl = dataSource.GetRssItemsForChannel(channelId);
            Assert.AreEqual(2, rssColl.Count);

            int i = 0;
            foreach (RssItem item in rssColl)
            {
                Assert.AreEqual(rss.Channels[0].Items[i].Title, item.Title);
                Assert.AreEqual(rss.Channels[0].Items[i].PubDate.ToString(), item.PubDate.ToString());
                Assert.AreEqual(rss.Channels[0].Items[i].Link.AbsoluteUri, item.Link.AbsoluteUri);
                Assert.AreEqual(rss.Channels[0].Items[i].Description, item.Description);
                Assert.AreEqual(rss.Channels[0].Items[i].Author, item.Author);
                i++;
            }
        }

        [Test]
        public void GetChannelIDForUrl()
        {
            int channelId = dataSource.InsertFeedDetails(rss.Channels[0]);
            int actualChannelId = dataSource.GetChannelIDForUrl(TestHelper.url);
            Assert.AreEqual(channelId, actualChannelId);
        }

        [Test]
        public void SaveUser()
        {
            int id = dataSource.SaveUser(TestHelper.CreateUser());
            Assert.GreaterThan(id, 0);
        }

        [Test]
        public void ConvertUserToSubSonicUser()
        {
            User user = TestHelper.CreateUser();
            MockSubSonic.DAL.User subUser = dataSource.ConvertUserObject(user);
            Assert.AreEqual(user.Name, subUser.Name);
            Assert.AreEqual(user.Email, subUser.Email);
            Assert.AreEqual(user.Password, subUser.Password);
        }

        [Test]
        public void SubscribeUser()
        {
            User user = TestHelper.CreateUser();
            int channelId = 1;
            dataSource.SaveUser(user);
            bool result = dataSource.Subscribe(user.Name, channelId);
            Assert.IsTrue(result);
        }

        [Test]
        public void GetUserIdForUserName()
        {
            User u = TestHelper.CreateUser();
            int userId = dataSource.SaveUser(u);
            int actualUserId = dataSource.GetUserIDForUsername(u.Name);
            Assert.AreEqual(userId, actualUserId);
        }

        [Test]
        public void GetChannelById()
        {
            int channelId = dataSource.InsertFeedDetails(rss.Channels[0]);
            RssChannel actualChan = dataSource.GetRssChannel(channelId);
            Assert.AreEqual(rss.Channels[0].Title, actualChan.Title);
        }

        [Test]
        public void GetChannelListForUser()
        {
            User user = TestHelper.CreateUser();
            int channelId = 1;
            int userid = dataSource.SaveUser(user);
            dataSource.Subscribe(user.Name, channelId);
            int[] list = dataSource.GetChannelListForUser(userid);
            Assert.AreEqual(1, list.Length);
            Assert.AreEqual(1, list[0]);
        }

        [Test]
        public void SaveFeedWithInvalidDate()
        {
            RssChannel rssChannel = rss.Channels[0];
            rssChannel.PubDate = Convert.ToDateTime("01/01/0001");
            //Returns a SubSonic object. 
            MockSubSonic.DAL.RssChannel subSonicChannelObject = dataSource.ConvertRssChannelObject(rssChannel);
            Assert.AreEqual(rssChannel.Title, subSonicChannelObject.Title);
            Assert.AreEqual(rssChannel.Link, subSonicChannelObject.Uri);
            Assert.AreEqual(rssChannel.Description, subSonicChannelObject.Description);
            Assert.AreEqual("01/01/1970 00:00:00", subSonicChannelObject.PubDate.ToString());
            Assert.IsTrue(subSonicChannelObject.IsNew); //it shouldn't save it.
        }
    }
}
